@@ -1,4 +1,4 @@
-# Copyright 2013-2018 CERN for the benefit of the ATLAS collaboration.
+# Copyright 2013-2020 CERN for the benefit of the ATLAS collaboration.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,6 +21,8 @@
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 # - Martin Barisits <martin.barisits@cern.ch>, 2019
 # - Luc Goossens <luc.goossens@cern.ch>, 2020
+# - Patrick Austin, <patrick.austin@stfc.ac.uk>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 #
 # PY3K COMPATIBLE
 
@@ -34,19 +36,19 @@ from rucio.common.types import InternalAccount, InternalScope
 from rucio.common.utils import api_update_return_dict
 
 
-def get_bad_replicas_summary(rse_expression=None, from_date=None, to_date=None):
+def get_bad_replicas_summary(rse_expression=None, from_date=None, to_date=None, vo='def'):
     """
     List the bad file replicas summary. Method used by the rucio-ui.
     :param rse_expression: The RSE expression.
     :param from_date: The start date.
     :param to_date: The end date.
-    :param session: The database session in use.
+    :param vo: the VO to act on.
     """
-    replicas = replica.get_bad_replicas_summary(rse_expression=rse_expression, from_date=from_date, to_date=to_date)
+    replicas = replica.get_bad_replicas_summary(rse_expression=rse_expression, from_date=from_date, to_date=to_date, filter={'vo': vo})
     return [api_update_return_dict(r) for r in replicas]
 
 
-def list_bad_replicas_status(state=BadFilesStatus.BAD, rse=None, younger_than=None, older_than=None, limit=None, list_pfns=False):
+def list_bad_replicas_status(state=BadFilesStatus.BAD, rse=None, younger_than=None, older_than=None, limit=None, list_pfns=False, vo='def'):
     """
     List the bad file replicas history states. Method used by the rucio-ui.
     :param state: The state of the file (SUSPICIOUS or BAD).
@@ -54,29 +56,31 @@ def list_bad_replicas_status(state=BadFilesStatus.BAD, rse=None, younger_than=No
     :param younger_than: datetime object to select bad replicas younger than this date.
     :param older_than:  datetime object to select bad replicas older than this date.
     :param limit: The maximum number of replicas returned.
+    :param vo: The VO to act on.
     """
     rse_id = None
     if rse is not None:
-        rse_id = get_rse_id(rse=rse)
+        rse_id = get_rse_id(rse=rse, vo=vo)
 
     replicas = replica.list_bad_replicas_status(state=state, rse_id=rse_id, younger_than=younger_than,
-                                                older_than=older_than, limit=limit, list_pfns=list_pfns)
+                                                older_than=older_than, limit=limit, list_pfns=list_pfns, vo=vo)
     return [api_update_return_dict(r) for r in replicas]
 
 
-def declare_bad_file_replicas(pfns, reason, issuer):
+def declare_bad_file_replicas(pfns, reason, issuer, vo='def'):
     """
     Declare a list of bad replicas.
 
     :param pfns: The list of PFNs.
     :param reason: The reason of the loss.
     :param issuer: The issuer account.
+    :param vo: The VO to act on.
     """
     kwargs = {}
-    if not permission.has_permission(issuer=issuer, action='declare_bad_file_replicas', kwargs=kwargs):
+    if not permission.has_permission(issuer=issuer, vo=vo, action='declare_bad_file_replicas', kwargs=kwargs):
         raise exception.AccessDenied('Account %s can not declare bad replicas' % (issuer))
 
-    issuer = InternalAccount(issuer)
+    issuer = InternalAccount(issuer, vo=vo)
 
     replicas = replica.declare_bad_file_replicas(pfns=pfns, reason=reason, issuer=issuer, status=BadFilesStatus.BAD)
 
@@ -89,19 +93,20 @@ def declare_bad_file_replicas(pfns, reason, issuer):
     return replicas
 
 
-def declare_suspicious_file_replicas(pfns, reason, issuer):
+def declare_suspicious_file_replicas(pfns, reason, issuer, vo='def'):
     """
     Declare a list of bad replicas.
 
     :param pfns: The list of PFNs.
     :param reason: The reason of the loss.
     :param issuer: The issuer account.
+    :param vo: The VO to act on.
     """
     kwargs = {}
-    if not permission.has_permission(issuer=issuer, action='declare_suspicious_file_replicas', kwargs=kwargs):
+    if not permission.has_permission(issuer=issuer, vo=vo, action='declare_suspicious_file_replicas', kwargs=kwargs):
         raise exception.AccessDenied('Account %s can not declare suspicious replicas' % (issuer))
 
-    issuer = InternalAccount(issuer)
+    issuer = InternalAccount(issuer, vo=vo)
 
     replicas = replica.declare_bad_file_replicas(pfns=pfns, reason=reason, issuer=issuer, status=BadFilesStatus.SUSPICIOUS)
 
@@ -115,16 +120,17 @@ def declare_suspicious_file_replicas(pfns, reason, issuer):
     return replicas
 
 
-def get_did_from_pfns(pfns, rse):
+def get_did_from_pfns(pfns, rse, vo='def'):
     """
     Get the DIDs associated to a PFN on one given RSE
 
     :param pfns: The list of PFNs.
     :param rse: The RSE name.
+    :param vo: The VO to act on.
     :returns: A dictionary {pfn: {'scope': scope, 'name': name}}
     """
-    rse_id = get_rse_id(rse=rse)
-    replicas = replica.get_did_from_pfns(pfns=pfns, rse_id=rse_id)
+    rse_id = get_rse_id(rse=rse, vo=vo)
+    replicas = replica.get_did_from_pfns(pfns=pfns, rse_id=rse_id, vo=vo)
 
     for r in replicas:
         for k in r.keys():
@@ -137,7 +143,7 @@ def list_replicas(dids, schemes=None, unavailable=False, request_id=None,
                   client_location=None, domain=None, signature_lifetime=None,
                   resolve_archives=True, resolve_parents=False,
                   updated_after=None,
-                  issuer=None):
+                  issuer=None, vo='def'):
     """
     List file replicas for a list of data identifiers.
 
@@ -154,17 +160,18 @@ def list_replicas(dids, schemes=None, unavailable=False, request_id=None,
     :param resolve_parents: When set to True, find all parent datasets which contain the replicas.
     :param updated_after: datetime object (UTC time), only return replicas updated after this time
     :param issuer: The issuer account.
+    :param vo: The VO to act on.
     """
     validate_schema(name='r_dids', obj=dids)
 
     # Allow selected authenticated users to retrieve signed URLs.
     # Unauthenticated users, or permission-less users will get the raw URL without the signature.
     sign_urls = False
-    if permission.has_permission(issuer=issuer, action='get_signed_url', kwargs={}):
+    if permission.has_permission(issuer=issuer, vo=vo, action='get_signed_url', kwargs={}):
         sign_urls = True
 
     for d in dids:
-        d['scope'] = InternalScope(d['scope'])
+        d['scope'] = InternalScope(d['scope'], vo=vo)
 
     replicas = replica.list_replicas(dids=dids, schemes=schemes, unavailable=unavailable,
                                      request_id=request_id,
@@ -199,7 +206,7 @@ def list_replicas(dids, schemes=None, unavailable=False, request_id=None,
         yield rep
 
 
-def add_replicas(rse, files, issuer, ignore_availability=False):
+def add_replicas(rse, files, issuer, ignore_availability=False, vo='def'):
     """
     Bulk add file replicas.
 
@@ -207,29 +214,32 @@ def add_replicas(rse, files, issuer, ignore_availability=False):
     :param files: The list of files.
     :param issuer: The issuer account.
     :param ignore_availability: Ignore the RSE blacklisting.
+    :param vo: The VO to act on.
 
     :returns: True is successful, False otherwise
     """
+    for v_file in files:
+        v_file.update({"type": "FILE"})  # Make sure DIDs are identified as files for checking
     validate_schema(name='dids', obj=files)
 
-    rse_id = get_rse_id(rse=rse)
+    rse_id = get_rse_id(rse=rse, vo=vo)
 
     kwargs = {'rse': rse, 'rse_id': rse_id}
-    if not permission.has_permission(issuer=issuer, action='add_replicas', kwargs=kwargs):
+    if not permission.has_permission(issuer=issuer, vo=vo, action='add_replicas', kwargs=kwargs):
         raise exception.AccessDenied('Account %s can not add file replicas on %s' % (issuer, rse))
-    if not permission.has_permission(issuer=issuer, action='skip_availability_check', kwargs=kwargs):
+    if not permission.has_permission(issuer=issuer, vo=vo, action='skip_availability_check', kwargs=kwargs):
         ignore_availability = False
 
-    issuer = InternalAccount(issuer)
+    issuer = InternalAccount(issuer, vo=vo)
     for f in files:
-        f['scope'] = InternalScope(f['scope'])
+        f['scope'] = InternalScope(f['scope'], vo=vo)
         if 'account' in f:
-            f['account'] = InternalAccount(f['account'])
+            f['account'] = InternalAccount(f['account'], vo=vo)
 
     replica.add_replicas(rse_id=rse_id, files=files, account=issuer, ignore_availability=ignore_availability)
 
 
-def delete_replicas(rse, files, issuer, ignore_availability=False):
+def delete_replicas(rse, files, issuer, ignore_availability=False, vo='def'):
     """
     Bulk delete file replicas.
 
@@ -237,59 +247,64 @@ def delete_replicas(rse, files, issuer, ignore_availability=False):
     :param files: The list of files.
     :param issuer: The issuer account.
     :param ignore_availability: Ignore the RSE blacklisting.
+    :param vo: The VO to act on.
 
     :returns: True is successful, False otherwise
     """
     validate_schema(name='r_dids', obj=files)
 
-    rse_id = get_rse_id(rse=rse)
+    rse_id = get_rse_id(rse=rse, vo=vo)
 
     kwargs = {'rse': rse, 'rse_id': rse_id}
-    if not permission.has_permission(issuer=issuer, action='delete_replicas', kwargs=kwargs):
+    if not permission.has_permission(issuer=issuer, vo=vo, action='delete_replicas', kwargs=kwargs):
         raise exception.AccessDenied('Account %s can not delete file replicas on %s' % (issuer, rse))
-    if not permission.has_permission(issuer=issuer, action='skip_availability_check', kwargs=kwargs):
+    if not permission.has_permission(issuer=issuer, vo=vo, action='skip_availability_check', kwargs=kwargs):
         ignore_availability = False
 
     for f in files:
-        f['scope'] = InternalScope(f['scope'])
+        f['scope'] = InternalScope(f['scope'], vo=vo)
 
     replica.delete_replicas(rse_id=rse_id, files=files, ignore_availability=ignore_availability)
 
 
-def update_replicas_states(rse, files, issuer):
+def update_replicas_states(rse, files, issuer, vo='def'):
     """
     Update File replica information and state.
 
     :param rse: The RSE name.
     :param files: The list of files.
     :param issuer: The issuer account.
+    :param vo: The VO to act on.
     """
+    for v_file in files:
+        v_file.update({"type": "FILE"})  # Make sure DIDs are identified as files for checking
     validate_schema(name='dids', obj=files)
 
-    rse_id = get_rse_id(rse=rse)
+    rse_id = get_rse_id(rse=rse, vo=vo)
 
     kwargs = {'rse': rse, 'rse_id': rse_id}
-    if not permission.has_permission(issuer=issuer, action='update_replicas_states', kwargs=kwargs):
+    if not permission.has_permission(issuer=issuer, vo=vo, action='update_replicas_states', kwargs=kwargs):
         raise exception.AccessDenied('Account %s can not update file replicas state on %s' % (issuer, rse))
     replicas = []
     for file in files:
         rep = file
         rep['rse_id'] = rse_id
-        rep['scope'] = InternalScope(rep['scope'])
+        rep['scope'] = InternalScope(rep['scope'], vo=vo)
         replicas.append(rep)
     replica.update_replicas_states(replicas=replicas)
 
 
-def list_dataset_replicas(scope, name, deep=False):
+def list_dataset_replicas(scope, name, deep=False, vo='def'):
     """
     :param scope: The scope of the dataset.
     :param name: The name of the dataset.
     :param deep: Lookup at the file level.
+    :param vo: The VO to act on.
 
     :returns: A list of dict dataset replicas
     """
 
-    scope = InternalScope(scope)
+    scope = InternalScope(scope, vo=vo)
 
     replicas = replica.list_dataset_replicas(scope=scope, name=name, deep=deep)
 
@@ -298,41 +313,70 @@ def list_dataset_replicas(scope, name, deep=False):
         yield r
 
 
-def list_dataset_replicas_vp(scope, name, deep=False):
+def list_dataset_replicas_bulk(dids, vo='def'):
+    """
+    :param dids: The list of did dictionaries with scope and name.
+    :param vo: The VO to act on.
+
+    :returns: A list of dict dataset replicas
+    """
+
+    validate_schema(name='dids', obj=dids)
+    names_by_scope = dict()
+    for d in dids:
+        if d['scope'] in names_by_scope:
+            names_by_scope[d['scope']].append(d['name'])
+        else:
+            names_by_scope[d['scope']] = [d['name'], ]
+
+    names_by_intscope = dict()
+    for scope in names_by_scope:
+        internal_scope = InternalScope(scope, vo=vo)
+        names_by_intscope[internal_scope] = names_by_scope[scope]
+
+    replicas = replica.list_dataset_replicas_bulk(names_by_intscope)
+
+    for r in replicas:
+        yield api_update_return_dict(r)
+
+
+def list_dataset_replicas_vp(scope, name, deep=False, vo='def'):
     """
     :param scope: The scope of the dataset.
     :param name: The name of the dataset.
     :param deep: Lookup at the file level.
+    :param vo: The vo to act on.
 
     :returns: If VP exists a list of dicts of sites, otherwise nothing
 
     NOTICE: This is an RnD function and might change or go away at any time.
     """
 
-    scope = InternalScope(scope)
+    scope = InternalScope(scope, vo=vo)
     for r in replica.list_dataset_replicas_vp(scope=scope, name=name, deep=deep):
         yield api_update_return_dict(r)
 
 
-def list_datasets_per_rse(rse, filters={}, limit=None):
+def list_datasets_per_rse(rse, filters={}, limit=None, vo='def'):
     """
     :param scope: The scope of the dataset.
     :param name: The name of the dataset.
     :param filters: dictionary of attributes by which the results should be filtered.
     :param limit: limit number.
     :param session: Database session to use.
+    :param vo: The VO to act on.
 
     :returns: A list of dict dataset replicas
     """
 
-    rse_id = get_rse_id(rse=rse)
+    rse_id = get_rse_id(rse=rse, vo=vo)
     if 'scope' in filters:
-        filters['scope'] = InternalScope(filters['scope'])
+        filters['scope'] = InternalScope(filters['scope'], vo=vo)
     for r in replica.list_datasets_per_rse(rse_id, filters=filters, limit=limit):
         yield api_update_return_dict(r)
 
 
-def add_bad_pfns(pfns, issuer, state, reason=None, expires_at=None):
+def add_bad_pfns(pfns, issuer, state, reason=None, expires_at=None, vo='def'):
     """
     Add bad PFNs.
 
@@ -341,33 +385,34 @@ def add_bad_pfns(pfns, issuer, state, reason=None, expires_at=None):
     :param state: One of the possible states : BAD, SUSPICIOUS, TEMPORARY_UNAVAILABLE.
     :param reason: A string describing the reason of the loss.
     :param expires_at: Specify a timeout for the TEMPORARY_UNAVAILABLE replicas. None for BAD files.
+    :param vo: The VO to act on.
 
     :param session: The database session in use.
 
     :returns: True is successful.
     """
     kwargs = {'state': state}
-    if not permission.has_permission(issuer=issuer, action='add_bad_pfns', kwargs=kwargs):
+    if not permission.has_permission(issuer=issuer, vo=vo, action='add_bad_pfns', kwargs=kwargs):
         raise exception.AccessDenied('Account %s can not declare bad PFNs' % (issuer))
 
-    issuer = InternalAccount(issuer)
+    issuer = InternalAccount(issuer, vo=vo)
 
     return replica.add_bad_pfns(pfns=pfns, account=issuer, state=state, reason=reason, expires_at=expires_at)
 
 
-def get_suspicious_files(rse_expression, younger_than=None, nattempts=None):
+def get_suspicious_files(rse_expression, younger_than=None, nattempts=None, vo='def'):
     """
     List the list of suspicious files on a list of RSEs
     :param rse_expression: The RSE expression where the suspicious files are located
     :param younger_than: datetime object to select the suspicious replicas younger than this date.
     :param nattempts: The number of time the replicas have been declared suspicious
+    :param vo: The VO to act on.
     """
-    replicas = replica.get_suspicious_files(rse_expression=rse_expression, younger_than=younger_than, nattempts=nattempts)
-
+    replicas = replica.get_suspicious_files(rse_expression=rse_expression, younger_than=younger_than, nattempts=nattempts, filter={'vo': vo})
     return [api_update_return_dict(r) for r in replicas]
 
 
-def set_tombstone(rse, scope, name, issuer):
+def set_tombstone(rse, scope, name, issuer, vo='def'):
     """
     Sets a tombstone on one replica.
 
@@ -375,12 +420,13 @@ def set_tombstone(rse, scope, name, issuer):
     :param scope: scope of the replica DID.
     :param name: name of the replica DID.
     :param issuer: The issuer account
+    :param vo: The VO to act on.
     """
 
-    rse_id = get_rse_id(rse)
+    rse_id = get_rse_id(rse, vo=vo)
 
-    if not permission.has_permission(issuer=issuer, action='set_tombstone', kwargs={}):
+    if not permission.has_permission(issuer=issuer, vo=vo, action='set_tombstone', kwargs={}):
         raise exception.AccessDenied('Account %s can not set tombstones' % (issuer))
 
-    scope = InternalScope(scope)
+    scope = InternalScope(scope, vo=vo)
     replica.set_tombstone(rse_id, scope, name)
