@@ -481,20 +481,23 @@ def update_transfer_state(external_host, transfer_id, state, logging_prepend_str
 
 
 @transactional_session
-def get_hops(source_rse_id, dest_rse_id, include_multihop=False, multihop_rses=None, session=None):
+def get_hops(source_rse_id, dest_rse_id, include_multihop=False, multihop_rses=None, limit_dest_schemes=None, session=None):
     """
     Get a list of hops needed to transfer date from source_rse_id to dest_rse_id.
     Ideally, the list will only include one item (dest_rse_id) since no hops are needed.
-    :param source_rse_id:      Source RSE id of the transfer.
-    :param dest_rse_id:        Dest RSE id of the transfer.
-    :param include_multihop:   If no direct link can be made, also include multihop transfers.
-    :param multihop_rses:      List of RSE ids that can be used for multihop.
-    :returns:                  List of hops in the format [{'source_rse_id': source_rse_id, 'source_scheme': 'srm', 'source_scheme_priority': N, 'dest_rse_id': dest_rse_id, 'dest_scheme': 'srm', 'dest_scheme_priority': N}]
-    :raises:                   NoDistance
+    :param source_rse_id:       Source RSE id of the transfer.
+    :param dest_rse_id:         Dest RSE id of the transfer.
+    :param include_multihop:    If no direct link can be made, also include multihop transfers.
+    :param multihop_rses:       List of RSE ids that can be used for multihop.
+    :param limit_dest_schemes:  List of destination schemes the matching scheme algorithm should be limited to for a single hop.
+    :returns:                   List of hops in the format [{'source_rse_id': source_rse_id, 'source_scheme': 'srm', 'source_scheme_priority': N, 'dest_rse_id': dest_rse_id, 'dest_scheme': 'srm', 'dest_scheme_priority': N}]
+    :raises:                    NoDistance
     """
+    if not limit_dest_schemes:
+        limit_dest_schemes = []
 
     # Check if there is a cached result
-    result = REGION_SHORT.get('get_hops_%s_%s' % (str(source_rse_id), str(dest_rse_id)))
+    result = REGION_SHORT.get('get_hops_%s_%s_%s' % (str(source_rse_id), str(dest_rse_id), ''.join(sorted(limit_dest_schemes))))
     if not isinstance(result, NoValue):
         return result
 
@@ -511,7 +514,7 @@ def get_hops(source_rse_id, dest_rse_id, include_multihop=False, multihop_rses=N
     if distance_graph.get(source_rse_id, {dest_rse_id: None}).get(dest_rse_id) is not None:
         # Check if there is a protocol match between the two RSEs
         try:
-            matching_scheme = rsemgr.find_matching_scheme(rse_settings_dest=__load_rse_settings(rse_id=dest_rse_id, session=session),
+            matching_scheme = rsemgr.find_matching_scheme(rse_settings_dest=limit_dest_schemes if limit_dest_schemes else __load_rse_settings(rse_id=dest_rse_id, session=session),
                                                           rse_settings_src=__load_rse_settings(rse_id=source_rse_id, session=session),
                                                           operation_src='third_party_copy',
                                                           operation_dest='third_party_copy',
@@ -522,7 +525,7 @@ def get_hops(source_rse_id, dest_rse_id, include_multihop=False, multihop_rses=N
                      'dest_scheme': matching_scheme[0],
                      'source_scheme_priority': matching_scheme[3],
                      'dest_scheme_priority': matching_scheme[2]}]
-            REGION_SHORT.set('get_hops_%s_%s' % (str(source_rse_id), str(dest_rse_id)), path)
+            REGION_SHORT.set('get_hops_%s_%s_%s' % (str(source_rse_id), str(dest_rse_id), ''.join(sorted(limit_dest_schemes))), path)
             return path
         except RSEProtocolNotSupported as error:
             if include_multihop:
@@ -570,7 +573,7 @@ def get_hops(source_rse_id, dest_rse_id, include_multihop=False, multihop_rses=N
                         continue
                     # Check if there is a compatible protocol pair
                     try:
-                        matching_scheme = rsemgr.find_matching_scheme(rse_settings_dest=__load_rse_settings(rse_id=out_v, session=session),
+                        matching_scheme = rsemgr.find_matching_scheme(rse_settings_dest=limit_dest_schemes if out_v == dest_rse_id and limit_dest_schemes else __load_rse_settings(rse_id=out_v, session=session),
                                                                       rse_settings_src=__load_rse_settings(rse_id=current_node, session=session),
                                                                       operation_src='third_party_copy',
                                                                       operation_dest='third_party_copy',
@@ -589,7 +592,7 @@ def get_hops(source_rse_id, dest_rse_id, include_multihop=False, multihop_rses=N
                     except RSEProtocolNotSupported:
                         pass
     if dest_rse_id in visited_nodes:
-        REGION_SHORT.set('get_hops_%s_%s' % (str(source_rse_id), str(dest_rse_id)), visited_nodes[dest_rse_id]['path'])
+        REGION_SHORT.set('get_hops_%s_%s_%s' % (str(source_rse_id), str(dest_rse_id), ''.join(sorted(limit_dest_schemes))), visited_nodes[dest_rse_id]['path'])
         return visited_nodes[dest_rse_id]['path']
     else:
         raise NoDistance()
@@ -686,19 +689,37 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
 
         dict_attributes = get_attributes(attributes)
 
-        # Check if the source and destination are blacklisted
+        # Check if the source and destination are blocked
         if source_rse_id in unavailable_read_rse_ids:
             continue
         if dest_rse_id in unavailable_write_rse_ids:
-            logging.warning('RSE %s is blacklisted for write. Will skip the submission of new jobs', dest_rse_name)
+            logging.warning('RSE %s is blocked for write. Will skip the submission of new jobs', dest_rse_name)
             continue
 
+        # parse source expression
+        source_replica_expression = dict_attributes.get('source_replica_expression', None)
+        if source_replica_expression:
+            try:
+                parsed_rses = parse_expression(source_replica_expression, session=session)
+            except InvalidRSEExpression as error:
+                logging.error("Invalid RSE exception %s: %s", source_replica_expression, str(error))
+                continue
+            else:
+                allowed_rses = [x['id'] for x in parsed_rses]
+                if source_rse_id not in allowed_rses:
+                    continue
+
         # Call the get_hops function to create a list of RSEs used for the transfer
         # In case the source_rse and the dest_rse are connected, the list contains only the destination RSE
         # In case of non-connected, the list contains all the intermediary RSEs
         list_hops = []
         try:
-            list_hops = get_hops(source_rse_id, dest_rse_id, include_multihop=core_config_get('transfers', 'use_multihop', default=False, expiration_time=600, session=session), multihop_rses=multihop_rses, session=session)
+            list_hops = get_hops(source_rse_id,
+                                 dest_rse_id,
+                                 include_multihop=core_config_get('transfers', 'use_multihop', default=False, expiration_time=600, session=session),
+                                 multihop_rses=multihop_rses,
+                                 limit_dest_schemes=transfers.get(req_id, {}).get('schemes', None),
+                                 session=session)
             if len(list_hops) > 1:
                 logging.debug('From %s to %s requires multihop: %s', source_rse_id, dest_rse_id, list_hops)
                 multihop = True
@@ -741,19 +762,6 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
             if rses and dest_rse_id not in rses:
                 continue
 
-            # parse source expression
-            source_replica_expression = dict_attributes.get('source_replica_expression', None)
-            if source_replica_expression:
-                try:
-                    parsed_rses = parse_expression(source_replica_expression, session=session)
-                except InvalidRSEExpression as error:
-                    logging.error("Invalid RSE exception %s: %s", source_replica_expression, str(error))
-                    continue
-                else:
-                    allowed_rses = [x['id'] for x in parsed_rses]
-                    if source_rse_id not in allowed_rses:
-                        continue
-
             # Get the source rse information
             if source_rse_id not in rses_info:
                 rses_info[source_rse_id] = rsemgr.get_rse_info(rse=source_rse_name,
@@ -837,9 +845,10 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                     source_url = re.sub('davs', 'gclouds', source_url)
                     source_url = re.sub('https', 'gclouds', source_url)
 
-                # IV - get external_host + strict_copy
+                # IV - get external_host + strict_copy + archive timeout
                 strict_copy = rse_attrs[dest_rse_id].get('strict_copy', False)
                 fts_hosts = rse_attrs[dest_rse_id].get('fts', None)
+                archive_timeout = rse_attrs[dest_rse_id].get('archive_timeout', None)
                 if source_sign_url == 'gcs':
                     fts_hosts = rse_attrs[source_rse_id].get('fts', None)
                 source_globus_endpoint_id = rse_attrs[source_rse_id].get('globus_endpoint_id', None)
@@ -926,7 +935,14 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                     transfers[req_id]['initial_request_id'] = req_id
                 if strict_copy:
                     transfers[req_id]['strict_copy'] = strict_copy
-
+                if archive_timeout and (rses_info[dest_rse_id]['rse_type'] == RSEType.TAPE
+                                        or rses_info[dest_rse_id]['rse_type'] == 'TAPE'):
+                    try:
+                        transfers[req_id]['archive_timeout'] = int(archive_timeout)
+                        logging.debug('Added archive timeout to transfer.')
+                    except ValueError:
+                        logging.warning('Could not set archive_timeout for %s. Must be integer.', dest_url)
+                        pass
             else:
                 current_schemes = transfers[req_id]['schemes']
                 # parse allow tape source expression, not finally version.
@@ -938,40 +954,7 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                     continue
 
                 # I - Check if there is already a transfer with a higher dest_scheme_priority
-                # I.1 - There is already a transfer queued with a higher dest_scheme_priority, skip it
-                if dest_scheme_priority > transfers[req_id]['dest_scheme_priority']:
-                    continue
-
-                # I.2 - The current scheme has a higher priority than the previous one. Need to recompute the destination URL
-                if dest_scheme_priority < transfers[req_id]['dest_scheme_priority']:
-                    transfers[req_id]['dest_scheme_priority'] = dest_scheme_priority
-                    dest_rse_id_key = 'write_%s_%s' % (dest_rse_id, destination_protocol)
-                    if dest_rse_id_key not in protocols:
-                        protocols[dest_rse_id_key] = rsemgr.create_protocol(rses_info[dest_rse_id], 'third_party_copy', destination_protocol)
-
-                    # I.2.1 - Get dest space token
-                    dest_spacetoken = None
-                    if protocols[dest_rse_id_key].attributes and \
-                       'extended_attributes' in protocols[dest_rse_id_key].attributes and \
-                       protocols[dest_rse_id_key].attributes['extended_attributes'] and \
-                       'space_token' in protocols[dest_rse_id_key].attributes['extended_attributes']:
-                        dest_spacetoken = protocols[dest_rse_id_key].attributes['extended_attributes']['space_token']
-
-                    # I.3.2 - Compute the destination url
-                    if rses_info[dest_rse_id]['deterministic']:
-                        dest_url = list(protocols[dest_rse_id_key].lfns2pfns(lfns={'scope': scope.external, 'name': name}).values())[0]
-                    else:
-                        # compute dest url in case of non deterministic
-                        # naming convention, etc.
-                        dsn = get_dsn(scope, name, dict_attributes.get('dsn', None))
-                        # DQ2 path always starts with /, but prefix might not end with /
-                        naming_convention = rse_attrs[dest_rse_id].get('naming_convention', None)
-                        dest_path = construct_surl(dsn, name, naming_convention)
-                        if rses_info[dest_rse_id]['rse_type'] == RSEType.TAPE or rses_info[dest_rse_id]['rse_type'] == 'TAPE':
-                            if retry_count or activity == 'Recovery':
-                                dest_path = '%s_%i' % (dest_path, int(time.time()))
-
-                        dest_url = list(protocols[dest_rse_id_key].lfns2pfns(lfns={'scope': scope.external, 'name': name, 'path': dest_path}).values())[0]
+                # Deprecated, not necessary anymore #3682
 
                 # II - Build the source URL
                 source_url = list(protocols[source_rse_id_key].lfns2pfns(lfns={'scope': scope.external, 'name': name, 'path': path}).values())[0]
@@ -1265,6 +1248,7 @@ def get_transfer_requests_and_source_replicas(total_workers=0, worker_number=0,
                     # We make the assumption that the hop is never made through TAPE
                     transfers[req_id]['file_metadata']['src_type'] = 'DISK'
                     transfers[req_id]['sources'] = [(source_rse_name, source_url, source_rse_id, 0, 0)]
+                    transfers[req_id]['bring_online'] = bring_online
         if req_id in reqs_no_source:
             reqs_no_source.remove(req_id)
         if req_id in reqs_only_tape_source:
@@ -1448,10 +1432,13 @@ def __load_distance_edges_node(rse_id, session=None):
         for distance in session.query(models.Distance).join(models.RSE, models.RSE.id == models.Distance.dest_rse_id)\
                                .filter(models.Distance.src_rse_id == rse_id)\
                                .filter(models.RSE.deleted == false()).all():
+            if distance.ranking is None:
+                continue
+            ranking = distance.ranking if distance.ranking >= 0 else 0
             if distance.src_rse_id in distance_graph:
-                distance_graph[distance.src_rse_id][distance.dest_rse_id] = distance.ranking
+                distance_graph[distance.src_rse_id][distance.dest_rse_id] = ranking
             else:
-                distance_graph[distance.src_rse_id] = {distance.dest_rse_id: distance.ranking}
+                distance_graph[distance.src_rse_id] = {distance.dest_rse_id: ranking}
         REGION_SHORT.set('distance_graph_%s' % str(rse_id), distance_graph)
         result = distance_graph
     return result
