@@ -1,20 +1,33 @@
-# Copyright European Organization for Nuclear Research (CERN)
+# Copyright 2014-2020 CERN for the benefit of the ATLAS collaboration.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
-# You may not use this file except in compliance with the License.
+# you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-# http://www.apache.org/licenses/LICENSE-2.0
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 #
 # Authors:
-# - Martin Barisits, <martin.barisits@cern.ch>, 2014-2017
-# - Mario Lassnig, <mario.lassnig@cern.ch>, 2014
-# - Andrew Lister, <andrew.lister@stfc.ac.uk>, 2019
-# - Brandon White, <bjwhite@fnal.gov>, 2019
-# - Hannes Hansen, <hannes.jakob.hansen@cern.ch>, 2019
+# - Martin Barisits <martin.barisits@cern.ch>, 2014-2019
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2014-2015
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2014
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Brandon White <bjwhite@fnal.gov>, 2019
+# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
+# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+
+import unittest
 
 from dogpile.cache import make_region
 from hashlib import sha256
 
+from rucio.common.config import config_get_bool
 from rucio.common.types import InternalAccount, InternalScope
 from rucio.common.utils import generate_uuid as uuid
 from rucio.common.config import config_get
@@ -34,20 +47,25 @@ from rucio.tests.common import rse_name_generator
 from rucio.tests.test_rule import create_files, tag_generator
 
 
-class TestJudgeRepairer():
+class TestJudgeRepairer(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls):
+        if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
+            cls.vo = {'vo': config_get('client', 'vo', raise_exception=False, default='tst')}
+        else:
+            cls.vo = {}
+
         # Add test RSE
         cls.rse1 = 'MOCK'
         cls.rse3 = 'MOCK3'
         cls.rse4 = 'MOCK4'
         cls.rse5 = 'MOCK5'
 
-        cls.rse1_id = get_rse_id(rse=cls.rse1)
-        cls.rse3_id = get_rse_id(rse=cls.rse3)
-        cls.rse4_id = get_rse_id(rse=cls.rse4)
-        cls.rse5_id = get_rse_id(rse=cls.rse5)
+        cls.rse1_id = get_rse_id(rse=cls.rse1, **cls.vo)
+        cls.rse3_id = get_rse_id(rse=cls.rse3, **cls.vo)
+        cls.rse4_id = get_rse_id(rse=cls.rse4, **cls.vo)
+        cls.rse5_id = get_rse_id(rse=cls.rse5, **cls.vo)
 
         # Add Tags
         cls.T1 = tag_generator()
@@ -64,8 +82,8 @@ class TestJudgeRepairer():
         add_rse_attribute(cls.rse5_id, "fakeweight", 0)
 
         # Add quota
-        cls.jdoe = InternalAccount('jdoe')
-        cls.root = InternalAccount('root')
+        cls.jdoe = InternalAccount('jdoe', **cls.vo)
+        cls.root = InternalAccount('root', **cls.vo)
         set_local_account_limit(cls.jdoe, cls.rse1_id, -1)
         set_local_account_limit(cls.jdoe, cls.rse3_id, -1)
         set_local_account_limit(cls.jdoe, cls.rse4_id, -1)
@@ -80,7 +98,7 @@ class TestJudgeRepairer():
         """ JUDGE REPAIRER: Test to repair a rule with 1 failed transfer (lock)"""
 
         rule_repairer(once=True)  # Clean out the repairer
-        scope = InternalScope('mock')
+        scope = InternalScope('mock', **self.vo)
         files = create_files(3, scope, self.rse4_id, bytes=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.from_sym('DATASET'), self.jdoe)
@@ -107,7 +125,7 @@ class TestJudgeRepairer():
         """ JUDGE REPAIRER: Test to repair a rule with 1 failed transfer (lock)"""
 
         rule_repairer(once=True)  # Clean out the repairer
-        scope = InternalScope('mock')
+        scope = InternalScope('mock', **self.vo)
         files = create_files(4, scope, self.rse4_id, bytes=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.from_sym('DATASET'), self.jdoe)
@@ -131,7 +149,7 @@ class TestJudgeRepairer():
         """ JUDGE REPAIRER: Test to repair a rule with 1 failed transfer (lock)"""
 
         rule_repairer(once=True)  # Clean out the repairer
-        scope = InternalScope('mock')
+        scope = InternalScope('mock', **self.vo)
         files = create_files(4, scope, self.rse4_id, bytes=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.from_sym('DATASET'), self.jdoe)
@@ -153,7 +171,7 @@ class TestJudgeRepairer():
 
     def test_repair_a_rule_with_missing_locks(self):
         """ JUDGE EVALUATOR: Test the judge when a rule gets STUCK from re_evaluating and there are missing locks"""
-        scope = InternalScope('mock')
+        scope = InternalScope('mock', **self.vo)
         files = create_files(3, scope, self.rse4_id)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.from_sym('DATASET'), self.jdoe)
@@ -191,7 +209,7 @@ class TestJudgeRepairer():
 
     def test_repair_a_rule_with_source_replica_expression(self):
         """ JUDGE EVALUATOR: Test the judge when a with two rules with source_replica_expression"""
-        scope = InternalScope('mock')
+        scope = InternalScope('mock', **self.vo)
         files = create_files(3, scope, self.rse4_id)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.from_sym('DATASET'), self.jdoe)
@@ -226,7 +244,7 @@ class TestJudgeRepairer():
         """ JUDGE REPAIRER: Test to repair a rule with only 1 rse whose transfers failed (lock)"""
 
         rule_repairer(once=True)  # Clean out the repairer
-        scope = InternalScope('mock')
+        scope = InternalScope('mock', **self.vo)
         files = create_files(4, scope, self.rse4_id, bytes=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.from_sym('DATASET'), self.jdoe)
@@ -255,7 +273,7 @@ class TestJudgeRepairer():
         """ JUDGE REPAIRER: Test to repair a rule with 1 failed transfer and flip to other rse(lock)"""
 
         rule_repairer(once=True)  # Clean out the repairer
-        scope = InternalScope('mock')
+        scope = InternalScope('mock', **self.vo)
         files = create_files(4, scope, self.rse4_id, bytes=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.from_sym('DATASET'), self.jdoe)
@@ -280,12 +298,12 @@ class TestJudgeRepairer():
         """ JUDGE REPAIRER: Test to repair a rule with only 1 rse whose site is blacklisted"""
 
         rse = rse_name_generator()
-        rse_id = add_rse(rse)
+        rse_id = add_rse(rse, **self.vo)
         update_rse(rse_id, {'availability_write': False})
         set_local_account_limit(self.jdoe, rse_id, -1)
 
         rule_repairer(once=True)  # Clean out the repairer
-        scope = InternalScope('mock')
+        scope = InternalScope('mock', **self.vo)
         files = create_files(4, scope, self.rse4_id, bytes=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.from_sym('DATASET'), self.jdoe)
